@@ -17,19 +17,14 @@ from django.core.exceptions import ImproperlyConfigured
 from django.db import connections
 from django.db.models import query
 from django.db.models.sql import Query, RawQuery, constants, subqueries
-from django.db.backends.signals import connection_created
 from django.utils.encoding import force_unicode
 
-import django
 from itertools import islice
-from pkg_resources import parse_version
-DJANGO_14 = (parse_version(django.get_version()) >= parse_version('1.4'))
-DJANGO_16 = django.VERSION[:2] >= (1,6)
 
 import restkit
 import pytz
 
-from salesforce import auth, models
+from salesforce import auth, models, DJANGO_14, DJANGO_16
 from salesforce.backend import compiler
 from salesforce.fields import NOT_UPDATEABLE, NOT_CREATEABLE
 
@@ -42,6 +37,8 @@ log = logging.getLogger(__name__)
 
 API_STUB = '/services/data/v28.0'
 
+sf_alias = getattr(settings, 'SALESFORCE_DB_ALIAS', 'salesforce')
+
 # Values of seconds are with 3 decimal places in SF, but they are rounded to
 # whole seconds for the most of fields.
 SALESFORCE_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%f+0000'
@@ -52,10 +49,11 @@ else:
 
 def quoted_string_literal(s, d):
 	"""
-	According to the SQL standard, this should be all you need to do to escape any kind of string.
+	SOQL requires single quotes to be escaped.
+	http://www.salesforce.com/us/developer/docs/soql_sosl/Content/sforce_api_calls_soql_select_quotedstringescapes.htm
 	"""
 	try:
-		return "'%s'" % (s.replace("'", "''"),)
+		return "'%s'" % (s.replace("'", "\\'"),)
 	except TypeError, e:
 		raise NotImplementedError("Cannot quote %r objects: %r" % (type(s), s))
 
@@ -85,7 +83,7 @@ def process_json_args(args):
 
 def reauthenticate():
 	auth.expire_token()
-	oauth = auth.authenticate(settings.DATABASES[settings.SALESFORCE_DB_ALIAS])
+	oauth = auth.authenticate(settings.DATABASES[sf_alias])
 	return oauth['access_token']
 
 def handle_api_exceptions(url, f, *args, **kwargs):
@@ -121,7 +119,7 @@ def handle_api_exceptions(url, f, *args, **kwargs):
 
 def prep_for_deserialize(model, record, using):
 	attribs = record.pop('attributes')
-	
+
 	mod = model.__module__.split('.')
 	if(mod[-1] == 'models'):
 		app_label = mod[-2]
@@ -129,7 +127,7 @@ def prep_for_deserialize(model, record, using):
 		app_label = getattr(model._meta, 'app_label')
 	else:
 		raise ImproperlyConfigured("Can't discover the app_label for %s, you must specify it via model meta options.")
-	
+
 	fields = dict()
 	for x in model._meta.fields:
 		if not x.primary_key:
@@ -145,7 +143,7 @@ def prep_for_deserialize(model, record, using):
 				fields[x.name] = field_val[:-1]  # Fix time e.g. "23:59:59.000Z"
 			else:
 				fields[x.name] = field_val
-	
+
 	return dict(
 		model	= '.'.join([app_label, model.__name__]),
 		pk		= record.pop('Id'),
@@ -205,7 +203,7 @@ class SalesforceQuerySet(query.QuerySet):
 		sql, params = compiler.SQLCompiler(self.query, connections[self.db], None).as_sql()
 		cursor = CursorWrapper(connections[self.db], self.query)
 		cursor.execute(sql, params)
-		
+
 		pfd = prep_for_deserialize
 		for res in python.Deserializer(pfd(self.model, r, self.db) for r in cursor.results):
 			yield res.object
@@ -225,7 +223,7 @@ class SalesforceRawQuery(RawQuery):
 	def _execute_query(self):
 		self.cursor = CursorWrapper(connections[self.using], self)
 		self.cursor.execute(self.sql, self.params)
-	
+
 	def __repr__(self):
 		return "<SalesforceRawQuery: %r>" % (self.sql % tuple(self.params))
 
@@ -235,10 +233,10 @@ class SalesforceQuery(Query):
 	"""
 	from salesforce.backend import aggregates
 	aggregates_module = aggregates
-	
+
 	def clone(self, klass=None, memo=None, **kwargs):
 		return Query.clone(self, klass, memo, **kwargs)
-	
+
 	def has_results(self, using):
 		q = self.clone()
 		compiler = q.get_compiler(using=using)
@@ -247,7 +245,7 @@ class SalesforceQuery(Query):
 class CursorWrapper(object):
 	"""
 	A wrapper that emulates the behavior of a database cursor.
-	
+
 	This is the class that is actually responsible for making connections
 	to the SF REST API
 	"""
@@ -255,22 +253,21 @@ class CursorWrapper(object):
 		"""
 		Connect to the Salesforce API.
 		"""
-		connection_created.send(sender=self.__class__, connection=self)
 		self.settings_dict = conn.settings_dict
 		self.query = query
 		self.results = []
 		self.rowcount = None
-	
+
 	@property
 	def oauth(self):
 		return auth.authenticate(self.settings_dict)
-	
+
 	def execute(self, q, args=None):
 		"""
 		Send a query to the Salesforce API.
 		"""
 		from salesforce.backend import base
-		
+
 		self.rowcount = None
 		if(isinstance(self.query, SalesforceQuery)):
 			response = self.execute_select(q, args)
@@ -284,12 +281,16 @@ class CursorWrapper(object):
 			response = self.execute_delete(self.query)
 		else:
 			raise base.DatabaseError("Unsupported query: %s" % self.query)
-		
+
 		body = response.body_string()
 		jsrc = force_unicode(body).encode(settings.DEFAULT_CHARSET)
-		
+
 		if(jsrc):
-			data = json.loads(jsrc)
+			# parse_float set to decimal.Decimal to avoid precision errors when
+			# converting from the json number to a float to a Decimal object
+			# on a model's DecimalField...converts from json number directly
+			# a Decimal object
+			data = json.loads(jsrc, parse_float=decimal.Decimal)
 			# a SELECT query
 			if('totalSize' in data):
 				self.rowcount = data['totalSize']
@@ -300,15 +301,15 @@ class CursorWrapper(object):
 			# something we don't recognize
 			else:
 				raise base.DatabaseError(data)
-			
+
 			if('count()' in q.lower()):
 				# COUNT() queries in SOQL are a special case, as they don't actually return rows
 				data['records'] = [{self.rowcount:'COUNT'}]
-			
+
 			self.results = self.query_results(data)
 		else:
 			self.results = []
-	
+
 	def execute_select(self, q, args):
 		processed_sql = q % process_args(args)
 		url = u'%s%s?%s' % (self.oauth['instance_url'], '%s/query' % API_STUB, urllib.urlencode(dict(
@@ -316,16 +317,16 @@ class CursorWrapper(object):
 		)))
 		headers = dict(Authorization='OAuth %s' % self.oauth['access_token'])
 		resource = get_resource(url)
-		
+
 		log.debug(processed_sql)
 		return handle_api_exceptions(url, resource.get, headers=headers)
-	
+
 	def query_more(self, nextRecordsUrl):
 		url = u'%s%s' % (self.oauth['instance_url'], nextRecordsUrl)
 		headers = dict(Authorization='OAuth %s' % self.oauth['access_token'])
 		resource = get_resource(url)
 		return handle_api_exceptions(url, resource.get, headers=headers)
-	
+
 	def execute_insert(self, query):
 		table = query.model._meta.db_table
 		url = self.oauth['instance_url'] + API_STUB + ('/sobjects/%s/' % table)
@@ -336,7 +337,7 @@ class CursorWrapper(object):
 		resource = get_resource(url)
 		log.debug('INSERT %s%s' % (table, post_data))
 		return handle_api_exceptions(url, resource.post, headers=headers, payload=json.dumps(post_data))
-	
+
 	def execute_update(self, query):
 		table = query.model._meta.db_table
 		# this will break in multi-row updates
@@ -351,12 +352,12 @@ class CursorWrapper(object):
 		headers['Content-Type'] = 'application/json'
 		post_data = extract_values(query)
 		resource = get_resource(url)
-		
+
 		log.debug('UPDATE %s(%s)%s' % (table, pk, post_data))
 		ret = handle_api_exceptions(url, resource.request, method='PATCH', headers=headers, payload=json.dumps(post_data))
 		self.rowcount = 1
 		return ret
-	
+
 	def execute_delete(self, query):
 		table = query.model._meta.db_table
 		## the root where node's children may itself have children..
@@ -373,10 +374,10 @@ class CursorWrapper(object):
 		headers = dict()
 		headers['Authorization'] = 'OAuth %s' % self.oauth['access_token']
 		resource = get_resource(url)
-		
+
 		log.debug('DELETE %s(%s)' % (table, pk))
 		return handle_api_exceptions(url, resource.delete, headers=headers)
-	
+
 	def query_results(self, results):
 		output = []
 		while True:
@@ -385,17 +386,17 @@ class CursorWrapper(object):
 
 			if results['done']:
 				break
-			
+
 			# http://www.salesforce.com/us/developer/docs/api_rest/Content/dome_query.htm#heading_2_1
 			response = self.query_more(results['nextRecordsUrl'])
 			jsrc = force_unicode(response.body_string()).encode(settings.DEFAULT_CHARSET)
-		
+
 			if(jsrc):
 				results = json.loads(jsrc)
 			else:
 				break
 		return output
-	
+
 	def __iter__(self):
 		return iter(self.results)
 
