@@ -20,11 +20,11 @@ DATABASES = {
 	# in order to protect private secret values from unintentional committing.
 	'salesforce': {
 		'ENGINE': 'salesforce.backend',
-		"CONSUMER_KEY" : '',
-		"CONSUMER_SECRET" : '',
-		'USER': '',
-		'PASSWORD': '',
-		'HOST': 'https://test.salesforce.com',
+		"CONSUMER_KEY" : '3MVG9JZ_r.QzrS7icihiIU063YpiIRjEFszVnMCZZmLC5KORG6fhJIxw5ahqgV3lDDCrV..PyA8Jr5JT4JzQk',
+		"CONSUMER_SECRET" : '228684408521557504',
+		'USER': 'djangosalesforce@gmail.com',
+		'PASSWORD': 'Cb6>pd8d-A',
+		'HOST': 'https://login.salesforce.com',
 	}
 }
 
